"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAgent = createAgent;
exports.useOrCreateAgent = useOrCreateAgent;
const proxy_agent_1 = require("proxy-agent");
const proxy_options_1 = require("./proxy-options");
const ssh_1 = require("./ssh");
class DevtoolsProxyAgent extends proxy_agent_1.ProxyAgent {
    constructor(proxyOptions) {
        super({
            ...proxyOptions,
            getProxyForUrl: (url) => this._getProxyForUrl(url),
        });
        this._getProxyForUrl = (url) => {
            if (!this._reqLockResolve || !this._req) {
                throw new Error('getProxyForUrl() called without pending request');
            }
            this._reqLockResolve();
            const req = this._req;
            this._req = undefined;
            this._reqLock = undefined;
            this._reqLockResolve = undefined;
            return (0, proxy_options_1.proxyForUrl)(this.proxyOptions, url, req);
        };
        this.proxyOptions = proxyOptions;
        if (proxyOptions.proxy && new URL(proxyOptions.proxy).protocol === 'ssh:') {
            this.sshAgent = new ssh_1.SSHAgent(proxyOptions);
        }
    }
    async initialize() {
        await this.sshAgent?.initialize();
    }
    async connect(req, opts) {
        if (this.sshAgent)
            return this.sshAgent;
        while (this._reqLock) {
            await this._reqLock;
        }
        this._req = req;
        this._reqLock = new Promise((resolve) => (this._reqLockResolve = resolve));
        return await super.connect(req, opts);
    }
    destroy() {
        this.sshAgent?.destroy();
        super.destroy();
    }
}
function createAgent(proxyOptions) {
    return new DevtoolsProxyAgent(proxyOptions);
}
function useOrCreateAgent(proxyOptions, target) {
    if ('createConnection' in proxyOptions) {
        return proxyOptions;
    }
    else {
        if (target !== undefined &&
            !(0, proxy_options_1.proxyForUrl)(proxyOptions, target)) {
            return undefined;
        }
        return createAgent(proxyOptions);
    }
}
//# sourceMappingURL=agent.js.map